/*********************************************************************
Copyright 1992 by Digital Press/Digital Equipment Corporation, Elias
Israel, and Erik Fortune

Permission to use, copy, modify, distribute, and sell this software
and its documentation for any purpose is hereby granted without fee,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation.  Digital Equipment Corporation and the
authors make no representations about the suitability of this software
for any purpose.  It is provided "as is" without express or implied
warranty.

DIGITAL EQUIPMENT CORPORATION AND THE AUTHORS DISCLAIM ALL WARRANTIES
WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL DIGITAL EQUIPMENT
CORPORATION OR THE AUTHORS BE LIABLE FOR ANY SPECIAL, INDIRECT OR
CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.

This material is reproduced with permission from _The X Window System
Server: X Version 11, Release 5_, by Elias Israel and Erik Fortune.
(Copyright 1992, Digital Press, Digital Equipment Corporation, One
Burlington Woods Drive, Burlington, MA 01803)
*********************************************************************/

#ifndef DDXSCREENS_H
#define	DDXSCREENS_H 1

	/* 
	 * There are two ways to get a device:
	 *    The user can specify the device to be opened on the
	 * command line.
	 *    The user can leave the device unspecified, in which
	 * case InitOutput tries to open each device in the
	 * array of known devices.   InitOutput opens the
	 * first 'ddxMaxScreens' available screens.
	 */

	/*
	 *    If the user chooses a device on the command line,
	 * ddxProcessCommandLine calls the Request function for 
	 * the specified device to initialize the ddxScreenRequest 
	 * structure for the specified device
	 */
typedef	void	(*ddxRequest)(
#ifdef __STDC__
	struct _ddxScreenDesc    *pDesc,
	struct _ddxScreenRequest *pReq
#endif
);
	 /*
	  *    The device specification option on the command line
	  * may be followed with a list of device specific settings 
	  * (a comma seperated list of names or name/value pairs).
	  * If device settings are provided, ddxProcessArgument calls 
	  * the SetValue function for the specified device for each 
	  * setting in turn.
	  */
typedef	int	(*ddxSetValue)(
#ifdef __STDC__
	struct _ddxScreenRequest *pReq,
	char			 *arg,
	char			 *value
#endif
);

	/*
	 *   If the user specifies the -help flag, invokes the server with
	 * an unknown argument or requests an unknown screen type, the server 
	 * prints a usage message.   The ddxProcessArgument routine prints
	 * a usage message for each common argument and then calls the 
	 * ddxScreenUseMsg for each known screen type to describe the device.
	 * The ddxScreenUseMsg routine should print a brief (about 30 
	 * characters) description of the device without any leading space
	 * or a trailing newline.
	 */
typedef void (*ddxScreenUseMsg)(
#ifdef __STDC__
	struct _ddxScreenDesc	*pDesc
#endif
);

	/*
	 *    If the user does not specify any devices on the command
	 * line, InitOutput calls the FindScreen routine for each known 
	 * device (until the number of requested devices reaches 
	 * ddxMaxScreens).    The FindScreen device routine should add 
	 * a request with default parameters for each available screen of 
	 * the associated type.   The FindScreen routine should use the common
	 * variables directly.
	 */
typedef	void	(*ddxFindScreen)(
#ifdef __STDC__
	struct _ddxScreenDesc *pDesc
#endif
);

	/*
	 *    Once we have a list of device requests (either user specified
	 * or automatically generated), we have to make sure that all of the 
	 * requested screens are present and available and that each request 
	 * is legal.   A probe function returns TRUE if the requested device 
	 * is available and the request is legal, otherwise it returns FALSE.
	 * InitOutput probes each requested device.
	 */
typedef Bool	(*ddxProbe)(
#ifdef __STDC__
	struct _ddxScreenRequest *pReq
#endif
);

	/*
	 *   The ScreenInit function for a request opens the specified 
	 * device and initializes the ScreenRec which describes it.
	 * InitOutput passes the ScreenInit function for a device in to
	 * the DIX AddScreens routine, which actually calls it.
	 */
typedef Bool	(*ddxScreenInit)(
#ifdef __STDC__
	int		  ndx,
	ScreenPtr	  pScreen,
	int		  argc,
	char		**argv
#endif
);

	/*
	 *   Each ddxScreenDesc structure lists some information about
	 * a possible display device.  The ddxProcessArgument routine
	 * uses the name and requestFunc to identify devices requested
	 * by the user.   If the user does not explicitly specify a device,
	 * the InitOutput routine calls the findScreenFunc for each known 
	 * screen type in turn to look for a usable screen of that type.
	 * It stops when it trys all screen types or when it finds
	 * ddxMaxScreens usable screens.
	 */
typedef struct _ddxScreenDesc {
	char		*name;
	ddxRequest	 requestFunc;
	ddxFindScreen	 findScreenFunc;
	ddxScreenUseMsg	 screenUseMsg;
} ddxScreenDesc;

	/*
	 *    Each ddxScreenRequest structure corresponds to a command
	 * line argument which chooses a device configuration. The 
	 * ddxProcessArgument routine uses the setValueFunc to interpret 
	 * device settings, and the InitOutput routine uses all of the other 
	 * fields to verify the request and initialize the server and the 
	 * device.
	 */
typedef struct _ddxScreenRequest {
	ddxProbe	 probeFunc;
	ddxScreenInit	 initFunc;
	ddxSetValue	 setValueFunc;

	int		 nFormats;
	PixmapFormatRec	*formats;

	ddxScreenDesc	*desc;
	pointer		 devPriv;
} ddxScreenReq;

extern	int		ddxMaxScreens;

extern	int		nRequestedScreens;
extern	ddxScreenReq	requestedScreens[MAXSCREENS];

extern	int		 nKnownScreens;
extern	ddxScreenDesc	*pKnownScreens[];

	/*
	 * If the server is running in a virtual screen (which might
	 *	be taken away), the os-specific DDX code calls the 
	 *	vsStateChanged routine whenever the state of a screen
	 *	changes.
	 */
typedef int	(*ddxVSState)(
#ifdef __STDC__
	ScreenRec		*pScreen,
	int			 gained
#endif
);

	/*
	 * The os-specific DDX code calls the ddxCleanupScreen
	 *	function to restore the display state when the
	 *	server exits abnormally.
	 */
typedef void	(*ddxCleanupScreen)(
#ifdef __STDC__
	ScreenRec	*pScreen
#endif
);

	/*
	 * InitOutput allocates a screen private index for the
	 * private information that is common to all screens.
	 * Each screen should fill it in.
	 */
typedef struct _ddxScreenPriv {
	ddxVSState		vsStateChanged;
	ddxCleanupScreen	cleanupScreen;
} ddxScreenPriv;

extern	int	ddxScreenPrivIndex;

extern	Bool ddxCursorOffScreen(/* ppScreen, pX, pY */);

#endif /* DDXSCREENS_H */
