/*********************************************************************
Copyright 1992 by Digital Press/Digital Equipment Corporation, Elias
Israel, and Erik Fortune

Permission to use, copy, modify, distribute, and sell this software
and its documentation for any purpose is hereby granted without fee,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation.  Digital Equipment Corporation and the
authors make no representations about the suitability of this software
for any purpose.  It is provided "as is" without express or implied
warranty.

DIGITAL EQUIPMENT CORPORATION AND THE AUTHORS DISCLAIM ALL WARRANTIES
WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL DIGITAL EQUIPMENT
CORPORATION OR THE AUTHORS BE LIABLE FOR ANY SPECIAL, INDIRECT OR
CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.

This material is reproduced with permission from _The X Window System
Server: X Version 11, Release 5_, by Elias Israel and Erik Fortune.
(Copyright 1992, Digital Press, Digital Equipment Corporation, One
Burlington Woods Drive, Burlington, MA 01803)
*********************************************************************/

#include "X.h"
#include "Xproto.h"
#include "screenint.h"
#include "input.h"
#include "cursor.h"
#include "misc.h"
#include "scrnintstr.h"
#include "servermd.h"

#include "ddxScreens.h"

int	ddxCommonGeneration = -1;
int	ddxScreenPrivIndex = -1;

/***====================================================================***/

	/*\
	 * DDX layer specific error routines
	 * ddxGiveUp()
	 *	The DIX layer calls ddxGiveUp to shut down the DDX layer 
	 *	when the server is about to exit normally.  It should 
	 *	reset any resources that are not reset by the normal
	 *	server shutdown process.  The DIX layer normally closes 
	 *	each input device and screen individually when the server
	 *	resets or exits.
	 *	
	 * AbortDDX() --
	 *	The OS layer calls AbortDDX to shut down the DDX layer
	 *	when the server encounters unrecoverable errors.  This 
	 *	function is called (among other things) when critical memory
	 *	cannot be allocated or reallocated.  If an allocation fails,
	 *	some server data structures may be corrupted or illegal, so
	 *	be careful.
	\*/
void
ddxGiveUp()
{
    osCloseInput();
    return;
}

void
AbortDDX()
{
int	i;
ddxScreenPriv	*priv;
ScreenPtr	 pScreen;

    osCloseInput();
    for (i=0;i<screenInfo.numScreens;i++) {
	pScreen = screenInfo.screens[i];
	priv = (ddxScreenPriv *)pScreen->devPrivates[ddxScreenPrivIndex].ptr;
	if ( priv && priv->cleanupScreen ) {
	    (*priv->cleanupScreen)(pScreen);
	}
    }
    return;
}

/***====================================================================***/

static Bool
FindSomeScreens()
{
int		 i;
ddxScreenDesc	*pDesc;

    for (i=0;(i<nKnownScreens)&&(nRequestedScreens<ddxMaxScreens);i++) {
	pDesc= pKnownScreens[i];
	if ((pDesc!=NULL)&&(pDesc->findScreenFunc!=NULL))
	    (*pDesc->findScreenFunc)(pDesc);
    }
    if (nRequestedScreens==0)	return(FALSE);
    else			return(TRUE);
}

/***====================================================================***/

static Bool
ProbeScreens()
{
int		 i;
Bool		 ok;
ddxScreenReq	*pReq;

    ok=	(nRequestedScreens>0);
    for (i=0;i<nRequestedScreens;i++) {
	pReq= &requestedScreens[i];
	if ((pReq->probeFunc==NULL)||(!(*pReq->probeFunc)(pReq))) {
	    ErrorF("Can't find a screen of type \"%s\"\n",(pReq->desc->name?
							      pReq->desc->name:
							      "<NONAME>"));
	    ok= FALSE;
	}
    }
    return(ok);
}

/***====================================================================***/

static Bool
AddRequestFormats(screenInfo,pReq)
ScreenInfo	*screenInfo;
ddxScreenReq	*pReq;
{
PixmapFormatRec	*pReqFmt,*pScrnFmt;
int		 nReqFmt,nScrFmt;
Bool		 add;
int		 origNFormats;

    origNFormats=	screenInfo->numPixmapFormats;

    for (nReqFmt=0;nReqFmt<pReq->nFormats;nReqFmt++) {
	add=	 TRUE;
	pReqFmt= &pReq->formats[nReqFmt];
	for (nScrFmt=0;nScrFmt<screenInfo->numPixmapFormats;nScrFmt++) {
	    pScrnFmt=	&screenInfo->formats[nScrFmt];
	    if ((pReqFmt->depth==pScrnFmt->depth)&&
		(pReqFmt->bitsPerPixel==pScrnFmt->bitsPerPixel)&&
		(pReqFmt->scanlinePad==pScrnFmt->scanlinePad)) {
		add= FALSE;
		break;
	    }
	}
	if (add) {
	    if (screenInfo->numPixmapFormats<MAXFORMATS) {
		screenInfo->formats[screenInfo->numPixmapFormats++]= *pReqFmt;
	    }
	    else {
		screenInfo->numPixmapFormats=	origNFormats;
		return(FALSE);
	    }
	}
    }
    return(TRUE);
}

/***====================================================================***/

void
InitOutput(screenInfo, argc, argv)
ScreenInfo	 *screenInfo;
int	  	  argc;
char		**argv;
{
int		  i;

    if ((nRequestedScreens==0)&&(!FindSomeScreens())) {
	FatalError("Couldn't find a screen to open\n");
	/* NOTREACHED */
    }
    if (!ProbeScreens()) {
	FatalError("Couldn't open one of the requested screens\n");
	/* NOTREACHED */
    }
    if ( ddxCommonGeneration != serverGeneration ) {
	ddxCommonGeneration= 	serverGeneration;
	ddxScreenPrivIndex=	AllocateScreenPrivateIndex();
    }

    screenInfo->imageByteOrder=		IMAGE_BYTE_ORDER;
    screenInfo->bitmapScanlineUnit=	BITMAP_SCANLINE_UNIT;
    screenInfo->bitmapScanlinePad=	BITMAP_SCANLINE_PAD;
    screenInfo->bitmapBitOrder=		BITMAP_BIT_ORDER;

    /* 
     * All servers must have a 1 bit visual for manipulating stipples
     * and masks
     */
    screenInfo->formats[0].depth=	1;
    screenInfo->formats[0].bitsPerPixel=1;
    screenInfo->formats[0].scanlinePad=	BITMAP_SCANLINE_PAD;
    screenInfo->numPixmapFormats=	1;

    for (i=0;i<nRequestedScreens;i++) {
	if (AddRequestFormats(screenInfo,&requestedScreens[i])) {
	    AddScreen(requestedScreens[i].initFunc, argc, argv);
	}
	else {
	    ErrorF("Couldn't create all necessary visuals\n");
	    ErrorF("Screen \"%s\" not added\n",requestedScreens[i].desc->name?
					       requestedScreens[i].desc->name:
					       "<NONAME>");
	}
    }
    return;
}
