/*********************************************************************
Copyright 1992 by Digital Press/Digital Equipment Corporation, Elias
Israel, and Erik Fortune

Permission to use, copy, modify, distribute, and sell this software
and its documentation for any purpose is hereby granted without fee,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation.  Digital Equipment Corporation and the
authors make no representations about the suitability of this software
for any purpose.  It is provided "as is" without express or implied
warranty.

DIGITAL EQUIPMENT CORPORATION AND THE AUTHORS DISCLAIM ALL WARRANTIES
WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL DIGITAL EQUIPMENT
CORPORATION OR THE AUTHORS BE LIABLE FOR ANY SPECIAL, INDIRECT OR
CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.

This material is reproduced with permission from _The X Window System
Server: X Version 11, Release 5_, by Elias Israel and Erik Fortune.
(Copyright 1992, Digital Press, Digital Equipment Corporation, One
Burlington Woods Drive, Burlington, MA 01803)
*********************************************************************/

#include <ctype.h>
#include <sys/types.h>
#include <sys/signal.h>
#include <sys/fcntl.h>
#define	NEED_EVENTS
#include <X11/X.h>
#include <X11/Xproto.h>
#include <X11/keysym.h>
#include "misc.h"
#include "inputstr.h"

#include "commonOS.h"
#include "xosInput.h"

#include "scrnintstr.h"
#include "ddxScreens.h"

static	unsigned	lastEventTime=	0;

void
InitInput(argc,argv)
int   argc;
char *argv[];
{
DevicePtr	ptr, kbd;

    /*
     * TBD (THIRD STAGE)
     * Write the OS-specific code to start using input devices in general
     * If your system supports an event queue, initialize it here.   You
     * should create and register devices here, but the DIX layer will 
     * explicitly initialize and open all devices as appropriate.
     * The DIX layer calls InitInput every server generation without explicitly 
     * "closing" input at the end of each generation, so pay attention to 
     * serverGeneration.
     *
     * If you support virtual screens, install handler functions
     * to be called whenever the server gains or loses a screen.  
     * xosVSGained and xosVSLost (above) are useful templates for such
     * a function.
     *
     * If your system input architecture provides a selectable file
     * descriptor, use AddEnabledDevice to tell the OS layer to check
     * it for events.
     *
     * If your system uses interrupts to indicate that input events are
     * available, set up your interrupt handlers here.
     */
    ptr=	AddInputDevice(xosPointerProc, TRUE);
    kbd=	AddInputDevice(xosKeyboardProc, TRUE);

    /*
     * TBD (FOURTH STAGE)
     *    If your system supports input devices other than a pointer and
     * keyboard, locate all devices to be used and initialize them here.
     *     If your system supports a generic device model (i.e. the pointer
     * and keyboard are not "special" in some way), locate all input devices 
     * here and then choose the core pointer and keyboard devices from the
     * list of available devices somehow.
     */

    RegisterPointerDevice(ptr);
    RegisterKeyboardDevice(kbd);

    /*
     * TBD (THIRD STAGE)
     *    If your system does not provide a shared memory event queue, you
     * can initialize the MI-provided generic event queue (mieq) here.
     * The MI Event Queue sets the input check pointers.    If you use it,
     * you must either report events asynchronously or set new input check 
     * pointers after you call mieqInit.
     */
#ifdef USE_MIEQ
    mieqInit(kbd,ptr);
#endif
    return;
}

void
osCloseInput()
{
    /* 
     * TBD (THIRD STAGE)
     * Perform any necessary operating system specific DDX layer cleanup 
     * for input devices in general.   The DIX layer will shut down each
     * device individually before it calls this function.
     */
    return;
}

void
ProcessInputEvents()
{
static	int alreadyHere = 0;
extern	int screenIsSaved;

    if (!alreadyHere) {
	if (screenIsSaved == SCREEN_SAVER_ON)
	    SaveScreens (SCREEN_SAVER_OFF, ScreenSaverReset);
	alreadyHere = 1;
	/*
	 * TBD (THIRD STAGE)
	 *    Read events from your system and pass them to the DIX layer.
	 *    If your system reports events asynchronously (using interrupts)
         * and you use the MI Generic Event Queue, you can simply call 
	 * mieqProcessInputEvents from here.
	 *    If your system has an event queue, you'll have to read each event
         * from the queue, move the cursor if necessary, and generate a 
         * corresponding X event, if necessary.   If you use the MI pointer
         * routines to control the cursor, you don't have to generate mouse 
	 * motion events -- simply call one of:
	 *    miPointerDeltaCursor( int dx, int dy, unsigned long time ), or
         *    miPointerAbsoluteCursor( int x, int y, unsigned long time )
         * as appropriate to move the cursor and generate the X event. 
	 * Note the MI pointer routines might call ProcessInputEvents to flush 
	 * any pending events before sending the event along, in which case the 
	 * mipointer EnqueueEvent routine must be reentrant with 
	 * ProcessInputEvents.   We use the static variable "alreadyHere" to 
	 * make sure that miPointer calls from WarpPointer correctly drain any 
	 * pending input while miPointer calls from this routine or 
	 * NewCurrentScreen do not loop indefinitely.
	 */
	alreadyHere = 0;
    }
    return;
}

    /*
     * TBD (THIRD STAGE)
     *    The display specific parts of this DDX-layer use the MI generic 
     * software cursor implementation by default.   The MI pointer code 
     * uses this function to queue events.
     *    If you also use the MI generic event queue, you can simply call 
     * mieqEnqueue from this routine or replace the pointer to this function
     * in each device directory with a pointer to mieqEnqueue.
     *    If you use the MI pointer code, but not the MI event queue, this 
     * function can simply send the event to DIX for processing.
     */
void
osEnqueueEvent (ev)
xEvent	*ev;
{
#ifdef USE_MIEQ
    mieqEnqueue( ev );
#else
DevicePtr pDev;

    pDev = LookupPointerDevice();
    if ( pDev ) {
	(*pDev->processInputProc)(ev,pDev,1);
    }
#endif
    return;
}

    /*
     * TBD (THIRD STAGE)
     *    The MI pointer code calls this routine to warp the cursor; it
     * should make sure that all pending events are handled before the
     * warp and that no asynchronous events come in while the warp is
     * taking place.
     *    If you use an event queue, this routine should flush any pending 
     * input events (see caution in ProcessInputEvents, above) and call
     * miPointerWarpCursor.
     *    If you get events asynchronously, this routine should temporarily
     * block further pointer events, call miPointerWarpCursor and reenable
     * pointer events.
     */
void
osWarpCursor( pScreen, x, y )
ScreenPtr   pScreen;
int         x, y;
{
    ProcessInputEvents();
    miPointerWarpCursor( pScreen, x, y );
    return;
}

    /*
     * TBD (FOURTH STAGE)
     *    The MI pointer code calls this routine whenever the pointer moves
     * from one queue to another.   Any events that come in after the call
     * to this routine should effect the new screen (pScreen).
     *    If you use the MI event queue, you should call mieqSwitchScreen.
     *    If you support multiple displays, this routine should flush the
     * pending input queue and call NewCurrentScreen to set the new screen
     * and coordinates.
     */
void
osNewEventScreen( pScreen )
{
#ifdef USE_MIEQ
    mieqSwitchScreen( pScreen );
#else
int x,y;

    ProcessInputEvents();
    miPointerPosition( &x, &y );
    NewCurrentScreen( pScreen, x, y );
#endif
    return;
}

    /*
     * The device specific calls this function when the screen saver times
     * out to reset the lastEventTime to the current time.
     */
void
osResetEventTime()
{
    lastEventTime= GetTimeInMillis();
    return;
}


    /*
     *    The OS layer calls this function to find out how long it has been 
     * since the last input event (or screen saver change).
     */
int
TimeSinceLastInputEvent()
{
   if (lastEventTime == 0) 
	lastEventTime=	GetTimeInMillis();
   return GetTimeInMillis() - lastEventTime;
}
