/*********************************************************************
Copyright 1992 by Digital Press/Digital Equipment Corporation, Elias
Israel, and Erik Fortune

Permission to use, copy, modify, distribute, and sell this software
and its documentation for any purpose is hereby granted without fee,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation.  Digital Equipment Corporation and the
authors make no representations about the suitability of this software
for any purpose.  It is provided "as is" without express or implied
warranty.

DIGITAL EQUIPMENT CORPORATION AND THE AUTHORS DISCLAIM ALL WARRANTIES
WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL DIGITAL EQUIPMENT
CORPORATION OR THE AUTHORS BE LIABLE FOR ANY SPECIAL, INDIRECT OR
CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.

This material is reproduced with permission from _The X Window System
Server: X Version 11, Release 5_, by Elias Israel and Erik Fortune.
(Copyright 1992, Digital Press, Digital Equipment Corporation, One
Burlington Woods Drive, Burlington, MA 01803)
*********************************************************************/

#include "X.h"
#include "Xproto.h"
#include "screenint.h"
#include "input.h"
#include "cursor.h"
#include "misc.h"
#include "scrnintstr.h"
#include "servermd.h"
#include "dix.h"
#include "../../mi/mipointer.h"

#include <sys/types.h>

#include "ddxScreens.h"

#define _NEED_XXX_POINTER_FUNCS
#define	_NEED_XXX_PRIVATE
#include "xxxDevInt.h"

    /*
     * This file includes the display-specific functions to open, initialize
     * and close your display device.   You will have to modify this file once
     * you start using your display hardware.
     */

static char *
xxxGetFramebuffer(pScreen)
ScreenPtr	pScreen;
{
int	bytesPerScreen;
char	*buf;

    /*
     * TBD (SECOND STAGE):
     *       If your device has an accessible framebuffer, put device 
     *    specific code to open the device framebuffer and return a pointer 
     *    to it here.   
     *       If your device does not have a directly accessible framebuffer, 
     *    you have quite a bit more work to do.   Chapter 15 talks about
     *    some of the steps you'll have to take to port to such a device.
     */
    bytesPerScreen=((XXX_BUFFER_WIDTH*XXX_BITS_PER_PIXEL)/8)*XXX_SCREEN_HEIGHT;
    buf=	(char *)xalloc(bytesPerScreen);
    if (buf==NULL) {
	    ErrorF("Couldn't allocate buffer for screen %d\n",pScreen->myNum);
    }
    return buf;
}

Bool
xxxScreenInit(ndx, pScreen, argc, argv)
int		  ndx;
ScreenPtr	  pScreen;
int		  argc;
char		**argv;
{
char		*buf;
xxxScreenPrivPtr		pPriv;

    pPriv=	(xxxScreenPrivPtr)xalloc(sizeof(xxxScreenPrivRec));
    if ( pPriv != NULL ) {
	bzero(pPriv,sizeof(xxxScreenPrivRec));
	pPriv->common.vsStateChanged = xxxVSStateChanged;
	pPriv->common.cleanupScreen = xxxCleanupScreen;
	/* initialize any display specific private fields here */
        pScreen->devPrivates[ddxScreenPrivIndex].ptr = (pointer)pPriv;
    }
    else {
	FatalError("Couldn't allocate private data for xxx\n");
	/* NOTREACHED */
    }

    if ((buf=xxxGetFramebuffer(pScreen))==NULL) {
	return(FALSE);
    }

    pScreen->SaveScreen=		xxxSaveScreen;

    pScreen->blackPixel=		XXX_BLACK_PIXEL;
    pScreen->whitePixel=		XXX_WHITE_PIXEL;

    if (!XXX_SCREEN_INIT( pScreen, buf, 
			 XXX_SCREEN_WIDTH, XXX_SCREEN_HEIGHT, 
			 XXX_X_RESOLUTION, XXX_Y_RESOLUTION, 
			 XXX_BUFFER_WIDTH)) {
	return(FALSE);
    }
    pPriv->CloseScreen = pScreen->CloseScreen;
    pScreen->CloseScreen = xxxCloseScreen;

#ifndef XXX_IS_MFB
    pScreen->InstallColormap = xxxInstallColormap;
    pScreen->UninstallColormap = xxxUninstallColormap;
    pScreen->ListInstalledColormaps = xxxListInstalledColormaps;
    pScreen->StoreColors = xxxStoreColors;
#endif

    /*
     * TBD (FOURTH STAGE):
     *    The miDC routines implement a generic software cursor.  If your
     *    display device provides a hardware cursor, you should replace
     *    the miDC routines with functions that support your hardware cursor.
     *    You can (and probably should) use the miPointer routines 
     *    (see ../../mi/mipointer.h) if you implement the functions in the 
     *    miPointerSpriteFuncRec for your hardware cursor.  This is a tuning
     *    step, so don't worry about it until your server is functional.
     */
    if (!miDCInitialize( pScreen, &xxxPointerScreenFuncs )) {
	ErrorF("Warning! miDCInitialize failed.\n");
	ErrorF("Cursor may not display correctly\n");
    }
    /* initialize colormap */
    if (!XXX_CREATE_DEF_COLORMAP(pScreen)) {
	return(FALSE);
    }
    return(TRUE);
}

Bool
xxxCloseScreen(index, pScreen)
int		index;
ScreenPtr	pScreen;
{
xxxScreenPrivPtr	pPriv;

    pPriv= xxxPriv(pScreen);
    if (pPriv->CloseScreen)
	(*pPriv->CloseScreen)(index, pScreen);
    /*
     * TBD (SECOND STAGE):
     *     Put device specific code to close the display here.  This function
     *     should free any private data associated with the display.  The
     *     DIX layer calls this function at the end of every server generation.
     */
    xfree(pPriv);
    return TRUE;
}

