/*********************************************************************
Copyright 1992 by Digital Press/Digital Equipment Corporation, Elias
Israel, and Erik Fortune

Permission to use, copy, modify, distribute, and sell this software
and its documentation for any purpose is hereby granted without fee,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation.  Digital Equipment Corporation and the
authors make no representations about the suitability of this software
for any purpose.  It is provided "as is" without express or implied
warranty.

DIGITAL EQUIPMENT CORPORATION AND THE AUTHORS DISCLAIM ALL WARRANTIES
WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL DIGITAL EQUIPMENT
CORPORATION OR THE AUTHORS BE LIABLE FOR ANY SPECIAL, INDIRECT OR
CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.

This material is reproduced with permission from _The X Window System
Server: X Version 11, Release 5_, by Elias Israel and Erik Fortune.
(Copyright 1992, Digital Press, Digital Equipment Corporation, One
Burlington Woods Drive, Burlington, MA 01803)
*********************************************************************/

#include "X.h"
#include "Xproto.h"
#include "screenint.h"
#include "input.h"
#include "cursor.h"
#include "misc.h"
#include "resource.h"
#include "windowstr.h"
#include "scrnintstr.h"
#include "servermd.h"
#include "dix.h"

#include "../../mi/mipointer.h"
#include "commonOS.h"
#include "ddxScreens.h"

#define _NEED_XXX_POINTER_FUNCS
#define	_NEED_XXX_PRIVATE
#include "xxxDevInt.h"

    /*
     * This file contains functions which detect or control assorted
     * characteristics of the display device.   You may have to modify
     * some of the functions in this file as you start using your
     * device.
     */

#define XXX_NUM_FORMATS (1+(XXX_DEPTH!=1))
PixmapFormatRec xxxFormats[] = {
	{1, 1, BITMAP_SCANLINE_PAD},
#if (XXX_DEPTH != 1)
	{XXX_DEPTH,XXX_BITS_PER_PIXEL,BITMAP_SCANLINE_PAD},
#endif
	{0, 0, 0}
};

/***====================================================================***/

Bool
xxxProbe(pReq)
ddxScreenReq	*pReq;
{
    /*
     * TBD (SECOND STAGE):
     * This function should decide if a screen of the type specified
     *     by pReq is present and available.    Return TRUE if
     *     a screen is available, FALSE otherwise.
     */
    return TRUE;
}

/***====================================================================***/

void
xxxSetValue(pReq,arg,value)
ddxScreenReq	*pReq;
char			*arg;
char			*value;
{
    /*
     * TBD (FOURTH STAGE):
     * If this device has any special settings, interpret them
     *     here.    Examples of device-specific settings might
     *     include a virtual screen size (for devices with hardware
     *     panning), a choice of several possible configurations 
     *     (e.g. 4 bit vs. 8 bit for VGA), or a board specification
     *     for devices that can have several coexisting devices.
     */
    ErrorF("xxx: screen request argument: %s%s%s ignored\n",arg,value?"=":"",
							   value?value:"");
    return;
}

/***====================================================================***/

    /*
     * This function prints a brief (about 30 characters) description of
     * this screen for a usage message.  The message should not have a
     * trailing carriage return or leading whitespace.
     */

static void
xxxScreenUseMsg(pDesc)
ddxScreenDesc *pDesc;
{
    /*
     * TBD (FOURTH STAGE)
     * This usage message is automatically generated and not very
     * descriptive.   Replace it with a brief description of your
     * device.
     */
    ErrorF("%d bit deep xxx device",XXX_DEPTH);
    return;
}

/***====================================================================***/

    /*
     * This function initializes the ddx screen request information
     * used by the common portion of this DDX layer to decide which
     * screens to open.  You should not have to modify this function
     * unless you add additional information to the screen request
     * (e.g. unit numbers, default visual, etc).
     */

static void
xxxRequestScreen(pDesc,pReq)
ddxScreenDesc	*pDesc;
ddxScreenReq	*pReq;
{
    pReq->probeFunc=	xxxProbe;
    pReq->initFunc=	xxxScreenInit;
    pReq->setValueFunc=	xxxSetValue;

    pReq->nFormats=	XXX_NUM_FORMATS;
    pReq->formats=	xxxFormats;

    pReq->desc=		pDesc;
    pReq->devPriv=	NULL;
    return;
}

/***====================================================================***/

    /*
     *    The common portion of this ddx layer calls xxxFindScreen when
     * the user starts the server without explicitly requesting a  display.
     * This routine should determine whether or not any displays of the type 
     * it supports are present and available and add requests for them if 
     * they are.   
     *    If you support several modes of the same device with different device
     * directories (for example, one-bit and eight-bit VGA modes), you may
     * have to modify this function to make sure that the different versions
     * don't "fight" for a single piece of hardware.   Don't worry about 
     * supporting multiple devices until your first device (or mode) is stable.
     */

static void
xxxFindScreen(pDesc)
ddxScreenDesc	*pDesc;
{
    if (nRequestedScreens<ddxMaxScreens) {
	xxxRequestScreen(pDesc,&requestedScreens[nRequestedScreens]);
	if (xxxProbe(&requestedScreens[nRequestedScreens]))
	    nRequestedScreens++;
    }
    return;
}

ddxScreenDesc	xxxDesc = {
	"xxx",	xxxRequestScreen,	xxxFindScreen,   xxxScreenUseMsg
};

/***====================================================================***/

Bool
xxxSaveScreen(pScreen, on)
ScreenPtr	pScreen;
Bool		on;
{
    /* 
     * TBD (FOURTH STAGE):
     *     If you can blank your display, you should add the code necessary
     * to turn screen blanking on or off where indicated below.   This is an
     * optimization and should be left until your server is fairly stable.
     * If you add code to blank and unblank the screen, remember to also
     * return TRUE -- if this function returns FALSE, the DIX layer will put
     * up a screen saver window.
     */
    if ( on == SCREEN_SAVER_FORCER )
	osResetEventTime();
    else if ( on == SCREEN_SAVER_ON ) {
	/* Blank the display here and return TRUE */
    }
    else {
	/* Turn off display blanking here and return TRUE */
    }
    return FALSE;
}

/***====================================================================***/

int
xxxCrossScreen( pScreen, on )
ScreenRec *pScreen;
int        on;
{
    /*
     * TBD (FOURTH STAGE)
     *    The MI pointer routines call this function when the cursor moves
     * from one screen to another.   If on is TRUE, the cursor has moved
     * onto pScreen, otherwise the cursor has left pScreen.
     *    Add code here to put up the cursor if it has entered the screen
     * or to take it down if it has left.
     */
    return FALSE;
}

miPointerScreenFuncRec xxxPointerScreenFuncs = {
        ddxCursorOffScreen,            /* CursorOffScreen */
        xxxCrossScreen,                /* CrossScreen */
        osWarpCursor,                  /* WarpCursor */
        osEnqueueEvent,                /* EnqueueEvent */
        osNewEventScreen,              /* NewEventScreen */
};

/***====================================================================***/

int
xxxVSStateChanged( pScreen, gained )
ScreenRec *pScreen;
int gained;
{
xxxScreenPrivPtr	pPriv;

    /* 
     * TBD (FOURTH STAGE)
     *    The OS-specific part of this DDX layer calls this function whenever
     * a screen gains or loses access to the physical hardware.
     *    If your system supports virtual screens (or you choose to 
     * support multiple screens on a single hardware display) you should
     * modify this function as indicated below.
     *    This is very much an extra feature and you should leave it until your
     * server is stable and reasonably fast.
     */
    pPriv= xxxPriv(pScreen);
    if (gained) {
	/*
	 * TBD (FOURTH STAGE)
	 * Add code to start using the display here.   Be sure to
	 * make all windows redraw.  One good way to do this is to
	 * create a fake window, map it over everything and destroy 
	 * it  -- all windows will get expose events.   
	 */
    }
    else {
	/*
	 * TBD (FOURTH STAGE)
	 * Add code to stop using the display here.   Make sure that
	 * no DDX functions affect the hardware after you lose the
	 * screen.    You can map a window above everything to stop
	 * most drawing requests, but you'll still have to look out
	 * for root window drawing requests that are not clipped by 
	 * children.  You should also make sure that your cursor and
	 * colormap routines note any changes but don't actually change
	 * the hardware (as you'll be changing it out from underneath
	 * whichever program owns the virtual screen).
	 */
    }
    return TRUE;
}

/***====================================================================***/

void
xxxCleanupScreen( pScreen )
ScreenRec *pScreen;
{
xxxScreenPrivPtr	priv;

   
    priv= xxxPriv(pScreen);
    /*
     * TBD (SECOND STAGE):
     * The OS-specific portion of this DDX layer calls this routine when
     * the server exits abnormally.   It should do the bare minimum that
     * is necessary to make the display hardware usable once the server
     * exits.
     */
    return;
}

